import { fileURLToPath } from "node:url";
import js from "@eslint/js";
import globals from "globals";
import eslintConfigPrettier from "eslint-config-prettier";
import eslintPluginEslintComments from "eslint-plugin-eslint-comments";
import eslintPluginPrettier from "eslint-plugin-prettier";
import eslintPluginImport from "eslint-plugin-import";
import eslintPluginN from "eslint-plugin-n";
import eslintPluginArrayFunc from "eslint-plugin-array-func";
import eslintPluginSecurity from "eslint-plugin-security";
import eslintPluginSonarjs from "eslint-plugin-sonarjs";

/**
 * @typedef {import("eslint").Linter.Config} Config
 */

/**
 * @param {Config} config
 * @returns {Config}
 */
function defineConfig(config) {
	return config;
}

export default [
	defineConfig({
		name: "@html-validate/eslint-config/language-options",
		languageOptions: {
			ecmaVersion: 2023,
			sourceType: "module",
			parserOptions: {
				ecmaFeatures: {
					globalReturn: true,
				},
			},
			globals: {
				...globals.es2023,
				...globals.node,
			},
		},
	}),

	defineConfig({
		name: "@html-validate/eslint-config/base",
		plugins: {
			"eslint-comments": eslintPluginEslintComments,
			prettier: eslintPluginPrettier,
			import: eslintPluginImport,
			n: eslintPluginN,
			"array-func": eslintPluginArrayFunc,
			security: eslintPluginSecurity,
			sonarjs: eslintPluginSonarjs,
		},
		settings: {
			"import/resolver": {
				[fileURLToPath(import.meta.resolve("eslint-import-resolver-node"))]: true,
				[fileURLToPath(import.meta.resolve("eslint-import-resolver-typescript"))]: true,
			},
		},
		rules: {
			...js.configs.recommended.rules,
			...eslintPluginEslintComments.configs.recommended.rules,
			...eslintConfigPrettier.rules,
			...eslintPluginPrettier.configs.recommended.rules,
			...eslintPluginImport.configs.errors.rules,
			...eslintPluginN.configs["recommended-module"].rules,
			...eslintPluginArrayFunc.configs.recommended.rules,
			...eslintPluginSecurity.configs.recommended.rules,
			...eslintPluginSonarjs.configs.recommended.rules,

			camelcase: "error",
			complexity: ["warn", 10],
			"consistent-return": "error",
			"consistent-this": "off",
			"dot-notation": "error",
			eqeqeq: ["error", "smart"],
			"max-depth": ["warn", 4],
			"new-cap": "error",
			"no-console": "warn",
			"no-dupe-class-members": "off",
			"no-eval": "error",
			"no-extend-native": "error",
			"no-implied-eval": "error",
			"no-loop-func": "error",
			"no-new": "error",
			"no-new-func": "error",
			"no-undef": "off",
			"no-unneeded-ternary": "error",
			"no-unused-vars": ["error", { ignoreRestSiblings: true, argsIgnorePattern: "^_" }],
			"no-var": "error",
			"no-warning-comments": "warn",
			"object-shorthand": "error",
			"prefer-const": "error",
			"prefer-rest-params": "error",
			"prefer-spread": "error",
			"prefer-template": "error",
			"prettier/prettier": "warn",
			radix: "error",
			strict: "off",
			yoda: "error",

			"eslint-comments/disable-enable-pair": ["error", { allowWholeFile: true }],
			"eslint-comments/require-description": [
				"error",
				{ ignore: ["eslint-enable", "eslint-env", "exported", "global", "globals"] },
			],
			"eslint-comments/no-unused-disable": "error",

			"import/default": "off",
			"import/extensions": ["error", "never", { json: "always" }],
			"import/newline-after-import": "error",
			"import/no-absolute-path": "error",
			"import/no-deprecated": "error",
			"import/no-dynamic-require": "error",
			"import/no-extraneous-dependencies": "error",
			"import/no-mutable-exports": "error",
			"import/no-named-default": "error",
			"import/no-useless-path-segments": "error",
			"import/order": "error",
			"import/no-named-as-default": "error",
			"import/no-named-as-default-member": "error",
			"import/no-duplicates": "error",

			/* this is checked by compiler and without additional configuration does not
			 * work with typescript */
			"n/no-missing-import": "off",
			"n/no-missing-require": "off",

			"security/detect-child-process": "off", // produces more noise than useful errors
			"security/detect-non-literal-fs-filename": "off", // html-validate reads files, don't want to acknowledge all occurrences
			"security/detect-non-literal-regexp": "error",
			"security/detect-non-literal-require": "error",
			"security/detect-object-injection": "off", // produces more noise than useful errors
			"security/detect-unsafe-regex": "error",

			"sonarjs/cognitive-complexity": "off", // already covered by native complexity rule
			"sonarjs/deprecation": "off", // already covered by @typescript-eslint/no-deprecated
			"sonarjs/function-return-type": "off", // overly broad and opinionated, let typescript deal with this
			"sonarjs/no-empty-test-file": "off", // could be useful but it does not handle it.each or similar constructs thus yields more false positives than its worth */
			"sonarjs/no-small-switch": "off", // prefer to use small switches when the intention is to all more cases later
			"sonarjs/no-unused-vars": "off", // already coveredby @typescript-eslint/no-unused-vars
			"sonarjs/unused-import": "off", // already covered by @typescript-eslint/no-unused-vars
			"sonarjs/prefer-nullish-coalescing": "off", // requires typescript and strictNullChecks, which is sane, but we also use @typescript-eslint/prefer-nullish-coalescing so this becomes redundant
			"sonarjs/prefer-single-boolean-return": "off", // prefer to use multiple returns even for booleans (looks better and can yield performance increase
			"sonarjs/no-control-regex": "off", // already covered by no-control-regexp
			"sonarjs/prefer-regexp-exec": "off", // prefer @typescript-eslint/prefer-regexp-exec
			"sonarjs/todo-tag": "off", // want to be able to leave todo tasks
			"sonarjs/void-use": "off", // used to silence warnings about unawaited promises
		},
	}),

	defineConfig({
		/* ensure all of these patterns are linted */
		name: "@html-validate/eslint-config/extensions",
		files: ["**/*.js", "**/*.cjs", "**/*.mjs"],
	}),

	defineConfig({
		/* mjs requires file extension */
		name: "@html-validate/eslint-config/esm",
		files: ["**/*.mjs"],
		rules: {
			"import/extensions": [
				"error",
				"always",
				{
					ignorePackages: true,
				},
			],
		},
	}),

	defineConfig({
		/* files which should lint even if project isn't build yet */
		name: "@html-validate/eslint-config/dist",
		files: ["*.d.ts", ".htmlvalidate.{js,mjs,cjs}", "bin/*.{js,mjs,cjs}"],
		rules: {
			"import/export": "off",
			"import/extensions": "off",
			"import/no-unresolved": "off",
		},
	}),
];
